
-- count salespeople
select count(*) from adventureworks.sales.salesperson

-- salespeople that sold product 866
SELECT DISTINCT SH.SalesPersonId FROM 
Sales.SalesOrderHeader SH 
JOIN Sales.SalesOrderDetail SD
ON SH.SalesOrderId = SD.SalesOrderId
WHERE SD.ProductID = 866
AND SH.SalesPersonId IS NOT NULL

-- list based on salespeople that sold product 866
SELECT SalesPersonID, 
CASE 
WHEN SalesPersonID IN 
(
SELECT SH.SalesPersonId FROM 
Sales.SalesOrderHeader SH 
JOIN Sales.SalesOrderDetail SD
ON SH.SalesOrderId = SD.SalesOrderId
WHERE SD.ProductID = 866
AND SH.SalesPersonId IS NOT NULL
)
THEN 'Make Call'
ELSE 'Relax'
END
FROM Sales.SalesPerson;

-- use a CTE to produce list
WITH SalesPeopleWithOrder866
AS
(
SELECT SH.SalesPersonId FROM 
Sales.SalesOrderHeader SH 
JOIN Sales.SalesOrderDetail SD
ON SH.SalesOrderId = SD.SalesOrderId
WHERE SD.ProductID = 866
AND SH.SalesPersonId IS NOT NULL
)
SELECT SalesPersonID, 
CASE 
WHEN SalesPersonID IN 
(SELECT * FROM SalesPeopleWithOrder866)
THEN 'Make Call'
ELSE 'Relax'
END
FROM Sales.SalesPerson

-- salepeople for product 744 (Subquery)
SELECT SalesPersonID ,
CASE 
WHEN SalesPersonID in
(
	SELECT SH.SalesPersonID FROM  Sales.SalesOrderHeader SH 
	WHERE SalesOrderID in
	(
		SELECT SalesOrderID FROM Sales.SalesOrderDetail
		WHERE ProductID = 744
)
)
THEN 'Make Call!'
ELSE 'Relax'
END AS Message
FROM Sales.SalesPerson;



-- salepeople for product 744 (CTE)
WITH 
DetailsFor744
AS
(
SELECT SalesOrderID FROM Sales.SalesOrderDetail
WHERE ProductID=744
),
MakeCall
AS
(
SELECT SalesPersonID FROM Sales.SalesOrderHeader
WHERE SalesOrderID in (SELECT SalesOrderID FROM DetailsFor744)
)
SELECT SalesPersonID,
CASE
WHEN SalesPersonID in (SELECT SalesPersonID FROM MakeCall) 
THEN 'Make Call!'
ELSE 'Relax'
END AS Message
FROM Sales.SalesPerson;


-- salepeople for product 744 (sub query)
SELECT SalesPersonID,
CASE
WHEN SalesPersonID in
(
SELECT SalesPersonID FROM Sales.SalesOrderHeader
WHERE SalesOrderID in
(SELECT SalesOrderID FROM Sales.SalesOrderDetail
WHERE ProductID = 744)
) 
THEN 'Make Call!'
ELSE 'Relax'
END
FROM Sales.SalesPerson
GO


CREATE FUNCTION MissingProdExp(@prod INT)
RETURNS TABLE
RETURN
WITH 
OrdersFor866
AS
(
SELECT SalesOrderID FROM Sales.SalesOrderHeader
WHERE SalesOrderID in (SELECT SalesOrderID FROM Sales.SalesOrderDetail
WHERE ProductID=866)
),
OrderTotalsFor866
AS
(
SELECT SalesOrderID, SUM(UnitPrice * OrderQty) AS Total From Sales.SalesOrderDetail
WHERE SalesOrderID in (SELECT SalesOrderID FROM OrdersFor866)
GROUP BY SalesOrderID
),
AvgOrdersFor866
AS
(
SELECT Avg(Total) AS Average FROM OrderTotalsFor866
),

AboveAvgOrdersFor866
AS
(
SELECT OT.SalesOrderID FROM OrderTotalsFor866 OT
JOIN AvgOrdersFor866 AS A ON 1 = 1
WHERE A.Average < OT.Total 
)
,
MakeCall
AS
(
SELECT SalesPersonID FROM Sales.SalesOrderHeader
WHERE SalesOrderID in (SELECT SalesOrderID FROM AboveAvgOrdersFor866)
)
--SELECT SalesPersonID FROM MakeCall
SELECT SalesPersonID,
CASE
WHEN SalesPersonID in (SELECT SalesPersonID FROM MakeCall) 
THEN 'Make Call!'
ELSE 'Relax'
END AS Message
FROM Sales.SalesPerson;


SELECT Count(*) from dbo.MissingProdExp(744)
WHERE Message = 'Make Call!'




