-- Subquery Concepts
-- select (single value subquery) as header
--	, expression as header
--	, header = expression
--	, expression header
-- from tname
-- where expression operator expression
-- or
-- where expression operator (single value subquery)
-- or
-- where expression operator 
-- 	(single value subquery OR
--            subqueries that return multiple values)
-- or
-- where EXISTS (subqueries that return multiple values)

-- To check out Perf use the "Show Execution Plan" (Ctrl+K)
SET STATISTICS IO ON
go

-- Single Value Subquery in the SELECT LIST
USE pubs
SELECT title_id, qty, 
	(SELECT sum(qty) FROM dbo.sales) AS TotalSales
FROM dbo.sales

-- Integer Math produces a 0 fraction
USE pubs
SELECT title_id, qty, 
		qty/(SELECT sum(qty) FROM dbo.sales)
			AS 'Percent of Total'
FROM dbo.sales

-- So we need to convert to preserve decimal places
USE pubs
SELECT title_id, qty, 
		(convert(decimal(5,3), qty)/
			(SELECT sum(qty) FROM dbo.sales)
				*100) AS 'Percent of Total'
FROM dbo.sales

-- Clean up the display to show only 5,3 in output
USE pubs
SELECT title_id, qty, 
	convert(decimal(5,3), 
		(convert(decimal(5,3), qty)/
			(SELECT sum(qty) FROM dbo.sales)
				*100)) AS 'Percent of Total'
FROM dbo.sales

select title_id, qty, CAST (((CAST (qty AS decimal(6,3))/
	(select sum(qty) from sales)) * 100) AS decimal(6,3)) AS PercentTotal
from sales
-- CAST and CONVERT 
-- convert(datatypeto, expressionfrom)
-- convert(decimal(5,3), qty)
-- cast (expressionfrom as datatypeto)
-- cast (qty as decimal(5,3))

-- In TEXT mode results (NOT Grid) you can Right Align Numerics
-- Tools, Options, Results Tab

-- If you really want a clean percentage and the actual % sign...
select title_id, qty, RIGHT('  ' + CAST(CAST (((CAST (qty AS decimal(5,3))/
	(select sum(qty) from sales)) * 100) AS decimal(2,0)) AS varchar) + '%', 3) AS [%]
from sales
order by 3 desc

-- If you want to RIGHT align within GRID mode...
SELECT s.title_id
, RIGHT(SPACE(15) 
		+ CONVERT(varchar(15), 
			CONVERT(decimal(5, 3), 
				(CONVERT(decimal(5, 3), s.qty) 
					/ (SELECT sum(qty) FROM sales)) 
					* 100)), 15) AS [PercentOfSales]
FROM dbo.sales AS s

-- Single Value Subquery in the where clause
USE pubs
SELECT title_id, qty, ord_date
FROM dbo.sales
WHERE ord_date = (SELECT max(ord_date)
	     FROM dbo.sales)

-- Subquery that returns multiple rows - in the WHERE clause

USE northwind
SELECT companyname
FROM dbo.customers
WHERE customerid IN
	(SELECT customerid
		 FROM dbo.orders
	 WHERE orderdate >'1/1/95')


-- Re-write as a Join?

USE northwind
SELECT  distinct companyname
FROM dbo.customers AS c
	JOIN dbo.orders AS o 
		ON c.customerid = o.customerid
 WHERE orderdate >'1/1/95'

--Re-write as a Correlated Subquery?

USE northwind
SELECT companyname
FROM dbo.customers AS c
WHERE EXISTS
	(SELECT *
	 FROM dbo.orders AS o
	 WHERE c.customerid = o.customerid 
		AND orderdate >'1/1/95')

-- Any other options? Group by...
SELECT c.companyname--, count(*) AS SalesCount
FROM dbo.customers AS c
	JOIN dbo.orders AS o
		ON c.customerid = o.customerid	 
WHERE o.orderdate > '1/1/95'
GROUP BY c.companyname
--ORDER BY SalesCount DESC



-- Correlated Subquery vs Join

USE pubs
SELECT title_id, (SELECT sum(qty) 
			FROM sales
		WHERE sales.title_id = titles.title_id) 
			AS TitleTotal
FROM titles

-- An update?
update t
	set title_id = (SELECT sum(qty) 
			FROM sales as s
			WHERE s.title_id = t.title_id) 
FROM titles as t

-- Comparable Join using GROUP BY

USE pubs
SELECT titles.title_id, sum(sales.qty) AS Total
FROM sales
	JOIN titles ON sales.title_id = titles.title_id
GROUP BY titles.title_id

-- Correlated Subquery from Queries Slide
USE northwind
SELECT orderid, customerid
FROM dbo.orders AS or1
WHERE 20 < (SELECT quantity
 	FROM dbo.[order details] od
	       	  WHERE or1.orderid = od.orderid
  	AND od.productid = 23)

-- Re-write as a Join?
USE northwind
SELECT or1.orderid, or1.customerid
FROM dbo.orders AS or1
	JOIN dbo.[order details] AS od
			ON or1.orderid = od.orderid
WHERE quantity > 20 AND od.productid = 23

